/*
 *	Copyright (C) 2009 Jonathan Naylor, G4KLX
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; version 2 of the License.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 */

#include "DStarRadioHeaderProcessor.h"
#include "LimitedPatternMatcher.h"
#include "GMSKModem.h"
#include "DStarScrambler.h"
#include "PatternMatcher.h"
#include "SoundReader.h"

#define	_CRT_SECURE_NO_WARNINGS
#include <cstdio>

#if !defined(__DSTARRECEIVER_H)
#define	__DSTARRECEIVER_H

enum DSR_STATE {
	DSR_SEARCH_BIT_SYNC,
	DSR_SEARCH_FRAME_SYNC,
	DSR_SEARCH_DATA_SYNC,
	DSR_PROCESS_HEADER,
	DSR_PROCESS_DV_VOICE,
	DSR_PROCESS_DD_DATA
};

class CDStarReceiver {
public:
	CDStarReceiver(ISoundReader* reader, FILE* fpInfo, FILE* fpAMBE, FILE* fpData);
	virtual ~CDStarReceiver();

	virtual void run();

	virtual void debug(bool set);

private:
	ISoundReader*              m_reader;
	CGMSKModem*                m_modem;
	CDStarRadioHeaderProcessor m_header;
	CLimitedPatternMatcher     m_frameMatcher;
	CLimitedPatternMatcher     m_dataMatcher;
	CPatternMatcher            m_endMatcher;
	CPatternMatcher            m_bitMatcher;
	CDStarScrambler            m_scrambler;
	DSR_STATE                  m_state;
	bool*                      m_buffer;
	unsigned int               m_length;
	unsigned int               m_count;
	unsigned int               m_dataLength;
	bool                       m_debug;
	FILE*                      m_fpInfo;
	FILE*                      m_fpAMBE;
	FILE*                      m_fpData;

	void addBit(bool bit);
	bool processFECSection();
	void interpretRadioHeader();
	void processDVVoice(const bool* data);
	void processDVSlowData(const bool* data);
	bool processDDData();
};

#endif
